==> nmrium-0.50.1.tgz: package/lib/component/modal/metaImportation/MetaImportationModal.js <==
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { Formik } from 'formik';
import { useState, useMemo, useRef, useEffect } from 'react';
import { DropZone } from 'react-science/ui';
import * as Yup from 'yup';
import { isMetaFile, linkMetaWithSpectra, parseMetaFile, TargetPathError, mapErrors, } from '../../../data/parseMeta';
import { useChartData } from '../../context/ChartContext';
import { useDispatch } from '../../context/DispatchContext';
import Button from '../../elements/Button';
import { CloseButton } from '../../elements/CloseButton';
import Label from '../../elements/Label';
import ReactTable from '../../elements/ReactTable/ReactTable';
import FormikInput from '../../elements/formik/FormikInput';
import FormikSelect from '../../elements/formik/FormikSelect';
import { useAlert } from '../../elements/popup/Alert/Context';
import { convertPathArrayToString } from '../../utility/convertPathArrayToString';
import { getSpectraObjectPaths } from '../../utility/getSpectraObjectPaths';
import { ModalStyles } from '../ModalStyle';
import { mapColumnToSelectItems } from './utils/mapColumnToSelectItems';
const styles = {
    container: {
        width: '100%',
        flex: 1,
        overflow: 'hidden',
        border: 'none',
        padding: '10px',
        display: 'flex',
        flexDirection: 'column',
    },
    column: {
        minWidth: '100px',
        maxWidth: '100px',
        overflow: 'hidden',
        textOverflow: 'ellipsis',
        whiteSpace: 'nowrap',
        width: '0px',
    },
};
const rowColors = {
    match: {
        activated: {
            backgroundColor: '#d6ffe6',
            color: '#28ba62',
        },
        hover: {
            backgroundColor: '#2dd36f',
            color: 'white',
        },
    },
    noMatch: {
        activated: {
            backgroundColor: '#ffd6db',
            color: '#cf3c4f',
        },
        hover: {
            backgroundColor: '#eb445a',
            color: 'white',
        },
    },
};
const validationSchema = Yup.object({
    source: Yup.string().required(),
    target: Yup.array(Yup.string()).min(1).required(),
});
function MetaImportationModal({ onClose, file }) {
    const alert = useAlert();
    const dispatch = useDispatch();
    const formRef = useRef(null);
    const [parseResult, setParseResult] = useState();
    const [compareResults, setCompareResults] = useState({});
    const [matches, setMatchesResults] = useState({});
    const { data } = useChartData();
    const { datalist, paths } = getSpectraObjectPaths(data);
    function handleParseFile(file) {
        void (async () => {
            const results = await parseMetaFile(file);
            setParseResult(results);
        })();
    }
    useEffect(() => {
        if (file) {
            handleParseFile(file);
        }
        return () => {
            setParseResult(null);
        };
    }, [file]);
    function handleDrop(files) {
        if (files[0]) {
            handleParseFile(files[0]);
        }
    }
    const errors = mapErrors(parseResult?.errors || []);
    const metaData = parseResult?.data || [];
    const columns = useMemo(() => {
        const fields = parseResult?.meta.fields || [];
        const columns = [
            { Header: '#', accessor: (_, index) => index + 1 },
        ];
        function getRowValue(val) {
            if (typeof val === 'string')
                return val;
            return val;
        }
        for (const fieldName of fields) {
            if (fieldName) {
                columns.push({
                    Header: fieldName,
                    accessor: (row) => getRowValue(row[fieldName]),
                    style: styles.column,
                });
            }
        }
        return columns;
    }, [parseResult?.meta.fields]);
    function handleLinkSpectra(fields) {
        const { source, target } = fields;
        if (data && parseResult) {
            try {
                const { compareResult, matches } = linkMetaWithSpectra({
                    source,
                    target,
                    spectra: data,
                    parseMetaFileResult: parseResult,
                });
                setMatchesResults(matches);
                if (Object.keys(compareResult).length > 0) {
                    setCompareResults(compareResult);
                }
                else {
                    alert.error(`No matches found:  Source field [${source}] => Target field [${target}]`);
                }
            }
            catch (error) {
                if (error instanceof TargetPathError) {
                    void formRef.current?.setFieldValue('target', null);
                    formRef.current?.setFieldError('target', '');
                }
                alert.error(error.message);
            }
        }
    }
    function handleActiveRow(data) {
        const record = compareResults[data.index] || null;
        if (record?.isDuplicated ||
            record?.spectraIDs.length > 0 ||
            errors?.[data.index]) {
            return true;
        }
        return false;
    }
    function handleRowStyle(data) {
        const record = compareResults[data.index] || null;
        if (record?.isDuplicated ||
            record?.spectraIDs.length > 1 ||
            errors?.[data.index]) {
            return rowColors.noMatch;
        }
        else if (record) {
            return rowColors.match;
        }
    }
    function handleImport() {
        dispatch({
            type: 'IMPORT_SPECTRA_META_INFO',
            payload: { spectraMeta: matches },
        });
        onClose?.();
    }
    return (_jsxs("div", { css: ModalStyles, style: { overflow: 'hidden' }, children: [_jsxs("div", { className: "header handle", children: [_jsx("span", { children: "Import Meta Information" }), _jsx(CloseButton, { title: "Close", onClick: onClose, className: "close-bt" })] }), _jsx("div", { style: styles.container, children: _jsx("div", { style: { flex: 1, width: '100%' }, children: !parseResult ? (_jsx(DropZone, { fileValidator: fileValidator, emptyDescription: "Drag and Drop CSV or tab-delimited file (.csv, .txt or .tsv)", onDrop: handleDrop })) : (_jsxs(_Fragment, { children: [_jsxs("div", { style: { height: '130px' }, children: [_jsx(Formik, { innerRef: formRef, initialValues: {
                                            source: null,
                                            target: '',
                                        }, onSubmit: handleLinkSpectra, validationSchema: validationSchema, children: _jsxs(_Fragment, { children: [_jsx(Label, { title: "Source field", style: { label: { width: '100px' } }, children: _jsx(FormikSelect, { name: "source", items: mapColumnToSelectItems(parseResult?.meta.fields || []), style: {
                                                            width: '300px',
                                                            padding: '5px',
                                                            margin: 0,
                                                        } }) }), _jsx(Label, { title: "Target field path", style: {
                                                        label: { width: '100px', padding: '20px 0px' },
                                                    }, children: _jsx(FormikInput, { name: "target", style: {
                                                            input: {
                                                                width: '300px',
                                                                textAlign: 'left',
                                                                padding: '5px',
                                                                fontSize: '14px',
                                                            },
                                                        }, placeholder: "Example: info.plus", datalist: datalist, mapOnChangeValue: (key) => paths?.[key] || null, mapValue: (paths) => convertPathArrayToString(paths) }) })] }) }), _jsx(Button.Done, { onClick: () => formRef.current?.submitForm(), children: "Link Spectra" })] }), _jsx("div", { style: { height: 'calc(100% - 160px)' }, children: _jsx(ReactTable, { columns: columns, data: metaData, approxItemHeight: 25, approxColumnWidth: 100, enableVirtualScroll: true, enableColumnsVirtualScroll: true, activeRow: handleActiveRow, rowStyle: handleRowStyle }) })] })) }) }), _jsx("div", { className: "footer-container", children: _jsx(Button.Done, { onClick: handleImport, disabled: Object.keys(matches).length === 0, children: "Import" }) })] }));
}
function fileValidator(file) {
    if (!isMetaFile(file)) {
        return {
            message: 'import a CSV or tab-delimited file',
            code: 'file - not - allow',
        };
    }
    return null;
}
export default MetaImportationModal;

==> nmrium-0.51.0.tgz: package/lib/component/modal/metaImportation/MetaImportationModal.js <==
import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { Dialog, DialogBody, DialogFooter } from '@blueprintjs/core';
import { Formik } from 'formik';
import { useState, useMemo, useRef, useEffect } from 'react';
import { DropZone } from 'react-science/ui';
import * as Yup from 'yup';
import { isMetaFile, linkMetaWithSpectra, parseMetaFile, TargetPathError, mapErrors, } from '../../../data/parseMeta';
import { useChartData } from '../../context/ChartContext';
import { useDispatch } from '../../context/DispatchContext';
import { useToaster } from '../../context/ToasterContext';
import Button from '../../elements/Button';
import Label from '../../elements/Label';
import ReactTable from '../../elements/ReactTable/ReactTable';
import FormikInput from '../../elements/formik/FormikInput';
import FormikSelect from '../../elements/formik/FormikSelect';
import { convertPathArrayToString } from '../../utility/convertPathArrayToString';
import { getSpectraObjectPaths } from '../../utility/getSpectraObjectPaths';
import { mapColumnToSelectItems } from './utils/mapColumnToSelectItems';
const styles = {
    container: {
        width: '100%',
        flex: 1,
        overflow: 'hidden',
        border: 'none',
        padding: '10px',
        display: 'flex',
        flexDirection: 'column',
    },
    column: {
        minWidth: '100px',
        maxWidth: '100px',
        overflow: 'hidden',
        textOverflow: 'ellipsis',
        whiteSpace: 'nowrap',
        width: '0px',
    },
};
const rowColors = {
    match: {
        activated: {
            backgroundColor: '#d6ffe6',
            color: '#28ba62',
        },
        hover: {
            backgroundColor: '#2dd36f',
            color: 'white',
        },
    },
    noMatch: {
        activated: {
            backgroundColor: '#ffd6db',
            color: '#cf3c4f',
        },
        hover: {
            backgroundColor: '#eb445a',
            color: 'white',
        },
    },
};
const validationSchema = Yup.object({
    source: Yup.string().required(),
    target: Yup.array(Yup.string()).min(1).required(),
});
export function MetaImportationModal(props) {
    const { isOpen, onCloseDialog, file } = props;
    if (!isOpen)
        return null;
    return (_jsx(Dialog, { isOpen: true, title: "Import Meta Information", style: { width: 900, height: 600 }, onClose: onCloseDialog, children: _jsx(InnerMetaImportationModal, { onCloseDialog, file }) }));
}
function InnerMetaImportationModal({ file, onCloseDialog, }) {
    const toaster = useToaster();
    const dispatch = useDispatch();
    const formRef = useRef(null);
    const [parseResult, setParseResult] = useState();
    const [compareResults, setCompareResults] = useState({});
    const [matches, setMatchesResults] = useState({});
    const { data } = useChartData();
    const { datalist, paths } = getSpectraObjectPaths(data);
    function handleParseFile(file) {
        void (async () => {
            const results = await parseMetaFile(file);
            setParseResult(results);
        })();
    }
    useEffect(() => {
        if (file) {
            handleParseFile(file);
        }
        return () => {
            setParseResult(null);
        };
    }, [file]);
    function handleDrop(files) {
        if (files[0]) {
            handleParseFile(files[0]);
        }
    }
    const errors = mapErrors(parseResult?.errors || []);
    const metaData = parseResult?.data || [];
    const columns = useMemo(() => {
        const fields = parseResult?.meta.fields || [];
        const columns = [
            { Header: '#', accessor: (_, index) => index + 1 },
        ];
        function getRowValue(val) {
            if (typeof val === 'string')
                return val;
            return val;
        }
        for (const fieldName of fields) {
            if (fieldName) {
                columns.push({
                    Header: fieldName,
                    accessor: (row) => getRowValue(row[fieldName]),
                    style: styles.column,
                });
            }
        }
        return columns;
    }, [parseResult?.meta.fields]);
    function handleLinkSpectra(fields) {
        const { source, target } = fields;
        if (data && parseResult) {
            try {
                const { compareResult, matches } = linkMetaWithSpectra({
                    source,
                    target,
                    spectra: data,
                    parseMetaFileResult: parseResult,
                });
                setMatchesResults(matches);
                if (Object.keys(compareResult).length > 0) {
                    setCompareResults(compareResult);
                }
                else {
                    toaster.show({
                        message: `No matches found:  Source field [${source}] => Target field [${target}]`,
                        intent: 'danger',
                    });
                }
            }
            catch (error) {
                if (error instanceof TargetPathError) {
                    void formRef.current?.setFieldValue('target', null);
                    formRef.current?.setFieldError('target', '');
                }
                toaster.show({
                    message: error.message,
                    intent: 'danger',
                });
            }
        }
    }
    function handleActiveRow(data) {
        const record = compareResults[data.index] || null;
        if (record?.isDuplicated ||
            record?.spectraIDs.length > 0 ||
            errors?.[data.index]) {
            return true;
        }
        return false;
    }
    function handleRowStyle(data) {
        const record = compareResults[data.index] || null;
        if (record?.isDuplicated ||
            record?.spectraIDs.length > 1 ||
            errors?.[data.index]) {
            return rowColors.noMatch;
        }
        else if (record) {
            return rowColors.match;
        }
    }
    function handleImport() {
        dispatch({
            type: 'IMPORT_SPECTRA_META_INFO',
            payload: { spectraMeta: matches },
        });
        onCloseDialog?.();
    }
    return (_jsxs(_Fragment, { children: [_jsx(DialogBody, { children: _jsx("div", { style: { width: '100%', height: '100%' }, onDrop: (e) => e.preventDefault(), children: !parseResult ? (_jsx(DropZone, { validator: fileValidator, emptyDescription: "Drag and Drop CSV or tab-delimited file (.csv, .txt or .tsv)", onDrop: handleDrop, noDragEventsBubbling: true })) : (_jsxs(_Fragment, { children: [_jsxs("div", { style: { height: '130px' }, children: [_jsx(Formik, { innerRef: formRef, initialValues: {
                                            source: null,
                                            target: [],
                                        }, onSubmit: handleLinkSpectra, validationSchema: validationSchema, children: _jsxs(_Fragment, { children: [_jsx(Label, { title: "Source field", style: { label: { width: '100px' } }, children: _jsx(FormikSelect, { name: "source", items: mapColumnToSelectItems(parseResult?.meta.fields || []), style: {
                                                            width: '300px',
                                                            padding: '5px',
                                                            margin: 0,
                                                        } }) }), _jsx(Label, { title: "Target field path", style: {
                                                        label: { width: '100px', padding: '20px 0px' },
                                                    }, children: _jsx(FormikInput, { name: "target", style: {
                                                            input: {
                                                                width: '300px',
                                                                textAlign: 'left',
                                                                padding: '5px',
                                                                fontSize: '14px',
                                                            },
                                                        }, placeholder: "Example: info.plus", datalist: datalist, mapOnChangeValue: (key) => paths?.[key] || key, mapValue: (paths) => convertPathArrayToString(paths) }) })] }) }), _jsx(Button.Done, { onClick: () => formRef.current?.submitForm(), children: "Link Spectra" })] }), _jsx("div", { style: { height: 'calc(100% - 160px)' }, children: _jsx(ReactTable, { columns: columns, data: metaData, approxItemHeight: 25, approxColumnWidth: 100, enableVirtualScroll: true, enableColumnsVirtualScroll: true, activeRow: handleActiveRow, rowStyle: handleRowStyle }) })] })) }) }), _jsx(DialogFooter, { children: _jsx(Button.Done, { onClick: handleImport, disabled: Object.keys(matches).length === 0, children: "Import" }) })] }));
}
function fileValidator(file) {
    if (!isMetaFile(file)) {
        return {
            message: 'import a CSV or tab-delimited file',
            code: 'file - not - allow',
        };
    }
    return null;
}
